<?php

define("VERSION", "2.0.2");

/**
 * Load and parse the config file, setting any defaults here if non-required
 * fields are not set
 *
 * @return string[]
 * @author Phil Burton <phil@pgburton.com>
 * @throws Exception when file is not readable
 * @throws Exception when ini file fails to parse
 * @throws Exception when url not found in ini file
 * @throws Exception when command not found in ini file
 * @throws Exception when pagesize is set incorrectly
 */
function getConfig()
{
    $config = false;

    if (!is_readable("config.ini")) {
        throw new Exception('File config.ini does not exist');
    }
    if (!$config = parse_ini_file("config.ini")) {
        throw new Exception('Could not parse ini file', 1);
    }
    if (!isset($config["url"])) {
        throw new Exception('`url` not found in ini file', 1);
    }
    if (!isset($config["command"])) {
        throw new Exception('`command` not found in ini file', 1);
    }

    // Set default page size if it's not in the config
    if (!isset($config["pagesize"])) {
        $config["pagesize"] = 3;
    } elseif (!is_int((int)$config["pagesize"]) || $config["pagesize"] < 1) {
        throw new Exception('Page size ' . $config["pagesize"]  . ' not valid in config', 1);
    }
    // hash the password
    if (isset($config['password'])) {
        $config['hash'] = hash("sha256", $config['password']);
    }
    return $config;
}

/**
 * Use bash to curl the url and grep the results.
 * Requires some exception handling.
 *
 * @return string[]
 * @author Phil Burton <phil@pburton.com>
 */
function curlGrepUrl($config, $search, $bounds)
{
    // var_dump($bounds);die();
    $grep = "curl -s '"
        . $config["url"]
        . "/?format=plain' | grep -i '"
        . implode(" ", $search)
        . "' |";
    if ($bounds["start"] > 1) {
        $grep .= "head -n" . $bounds["end"] . " | ";
    }
    $grep .= "tail -n" . $config["pagesize"] . " | "
        . " sed 's/^/" . $config["escape"]
        . ";:a;N;$!ba;s/\\n/ " . $config["url"] . "/g'";
    $stuff = shell_exec($grep);

    if (!isset($stuff)) {
        return false;
    }
    return explode("\n", substr($stuff, 0, -1));
}

/**
 * Use curl and grep and wc to get the total count
 *
 * @return string[]
 * @author Phil Burton <phil@pburton.com>
 */
function getCount($config, $search)
{
    // var_dump($bounds);die();
    $grep = "curl -s '"
        . $config["url"]
        . "/?format=plain' | grep -i '"
        . implode(" ", $search)
        . "' | sed 's/^/" . $config["escape"]
        . ";:a;N;$!ba;s/\\n/ " . $config["url"] . "/g' | wc -l";
    $count = shell_exec($grep);
    if (!isset($count)) {
        return 0;
    }
    return trim($count);
}

/**
 * Return the help text
 *
 * @param  StdClass[] $config
 * @return string
 * @author Phil Burton <phil@pgburton.com>
 */
function getHelp($config)
{
    return "Usage: !" . $config["command"] . " [query] #[page] | "
        . "e.g: !" . $config["command"] . " intense | "
        . "!" . $config["command"] . " intense marcus | "
        . "!" . $config["command"] . " intense marcus #2 (returns the second page of results)";
}

/**
 * Get the version string
 *
 * @param  stdClass[] $config
 * @return string
 * @author Phil Burton <phil@pgburton.com>
 */
function getVersion($config)
{
    return $config["command"] . " v" . VERSION;
}

/**
 * Output the given string, adding a new line at the end
 * If pieces is an array, use the glue to implode and output a single string
 *
 * EXIT
 *
 * @param  string|string[] $pieces
 * @param  string $glue
 * @param  bool $sderr
 * @author Phil Burton <phil@pgburton.com>
 */
function output($pieces, $glue = " - ", $stderr = false)
{
    $out = "";
    if (!is_array($pieces)) {
        $out = $pieces . "\n";
    } else {
        $out = implode($glue, $pieces) . "\n";
    }

    if ($stderr) {
        fwrite(STDERR, $out . "\n");
    } else {
        echo $out;
    }

    exit;
}

/**
 * Read stdin and return as a string
 *
 * @return string
 * @author Phil Burton <phil@pgburton.com>
 */
function read_stdin()
{
    $fr = fopen("php://stdin", "r");
    $input = fgets($fr, 128);
    $input = rtrim($input);
    fclose($fr);
    return $input;
}

/**
 * Get the start and end index positions for the required page and
 * page size
 *
 * @param  int $page
 * @param  int $pagesize
 * @return int[]
 * @author Phil Burton <phil@pgburton.com>
 */
function getPaginationBounds($page, $pagesize)
{
    return [
        "start" => ($page - 1) * $pagesize +1,
        "end" => (($page - 1) * $pagesize) + ($pagesize - 1) +1
    ];
}

/**
 * Loop through the stdin and parse the arguments given
 * If we hit a "special" argument then run the specific code and exit
 * Otherwise we set the page and search terms and return them as an array
 *
 * @param  $config
 * @return mixed[]
 * @author Phil Burton <phil@pgburton.com>
 */
function getFormattedArguments($config)
{
    $args = explode(" ", read_stdin());

    if (!isset($args) || empty($args)) {
        throw new Exception("No arguments found");
    }
    // Loop the raw arguments and create our formated argument array
    $formatArgs = [];
    $formatArgs["search"] = [];
    foreach ($args as $arg) {
        // Switch to check for special arguments
        // output should exit but incase it errors we double break
        switch ($arg) {
            case "--version":
            case "-v": // Flow through
                output(getVersion($config));
                break 2;
            case "--help":
            case "-h": // Flow through
                output(getHelp($config));
                break 2;
        }

        // Check to see if we have a page specified
        if (strpos($arg, "#") === 0) {
            $formatArgs["page"] = substr($arg, 1);
            continue;
        }

        // If we reach here, then the argument is a search term
        $formatArgs["search"][] = $arg;
    }

    // If we reach this point we must have at least one search term
    if (empty($formatArgs["search"])) {
        throw new Exception("Please Enter a search term");
    }

    if (!isset($formatArgs["page"])) {
        $formatArgs["page"] = 1;
    } elseif (!is_int((int)$formatArgs["page"]) || $formatArgs["page"] < 1) {
        throw new Exception("Page number `" . $formatArgs["page"] . "` is not valid");
    }

    return $formatArgs;
}

/*********************************
 * ENTRY POINT FOR FUNCTION HERE *
 *********************************/
try {
    $config = getConfig();
    $arguments = getFormattedArguments($config);

    // parse the arguments
    $page = $arguments["page"];
    $bounds = getPaginationBounds($page, $config["pagesize"]);
    $count = getCount($config, $arguments["search"]);
    $results = curlGrepUrl($config, $arguments["search"], $bounds);
    if (empty($results)) {
        output("No results found");
    }
    $out = [];
    foreach ($results as $index => $result) {
        $out[] = $index + 1 . ": " . $result;
    }

    if ($count > 3) {
        $out[] = "results (" . $count . ")";
    }

    output($out);

} catch (Exception $e) {
    output($e->getMessage(), " - ", ($e->getCode() == 1));
}
